/*
	BasketObserver
*/

import java.applet.*;
import java.awt.*;
import java.io.*;
import java.net.*;
import java.util.*;

// BasketObservers are applets that contact baskets (points where EGGS report to)
// and that display certain information about the EGGData collected there

public class BasketObserver extends Applet implements Runnable {

	//////////////////////// Static variables (Class variables)
	static final boolean	debug = false;			// set to true for debug information
	static final int		PAINT_CURRENT 	= 1;	// display current values
	static final int		PAINT_HISTORY	= 2;	// display history of values
	static final int		PAINT_HISTORY60	= 3;	// display history of values
	static final int		PAINT_COHERENCE	= 4;	// display coherence of values
	static final int		missingVal	= 999;		// missing values
	//////////////////////// network variables
	String				BasketAdress = "http://noosphere.princeton.edu/cgi-bin/eggdatareq.pl";	// URL of Basket data server

	//////////////////////// applet variables
	Thread 				thread  = null;	// main threat that performs the animations
	//////////////////////// representation variables
	Hashtable			eggs;			// Array of Eggs in the Basket, hashed by label
	String				source;			// Source File of Animations (raw CGI respons)
	int 				samp_rec = 0;	// samples per record
	int 				sec_rec  = 0; 	// seconds per record
	int 				rec_pkt  = 0;	// record per packet
	int 				trialsz  = 0; 	// Trials per sample
	int 				nummEggs = 0; 	// Eggs reporting
	Date				startDate;		// StartDate of Request
	Date				endDate;		// EndDate of request

	//////////////////////// GUI Variables
	Panel				WaitPanel;
	Panel				ErrorPanel;
	Panel				TimePanel;
	Panel				EggPanel;
	Panel				ButtonPanel;
	Panel				CoherePanel;
	Panel				MainPanel;
	Button				CurrentButton;
	Button				HistoryButton;
	Button				History60Button;
	Button				CohereButton;
	Color				bgColor;			// main background color;
	Color				fgColor;			// main foreground color;
	int					curr_display;
	AudioClip 			soundtrack = null;

	////////////////////////////////////////// main applet methods
	// initialize applet
	public void init(){
		if(debug) new debug("init entered");

		eggs = new Hashtable();

		initColors();

        setFont(new Font("Courier", Font.PLAIN, 10));
		this.setLayout(new BorderLayout(10,10));

		TimePanel = new Panel();

		CoherePanel = new Panel();

		EggPanel = new Panel();
		EggPanel.setLayout(new GridLayout(2, 3, 10, 10));

		WaitPanel = new Panel();
		WaitPanel.setForeground(Color.white);
		WaitPanel.setBackground(Color.black);
		Label l = new Label("Adjusting screen layout, please wait");
		l.setForeground(Color.white);
		l.setBackground(Color.black);
		WaitPanel.add(l);


		ErrorPanel = new Panel();
		ErrorPanel.setForeground(Color.white);
		ErrorPanel.setBackground(Color.darkGray);
		Label l2 = new Label("Connection too slow. Applet has quit.");
		l2.setForeground(Color.white);
		l2.setBackground(Color.darkGray);
		ErrorPanel.add(l2);

		MainPanel = new Panel();
		MainPanel.setLayout(new CardLayout());
		MainPanel.add("one",EggPanel);
		MainPanel.add("two",CoherePanel);
		MainPanel.add("three",WaitPanel);
		MainPanel.add("four",ErrorPanel);

		ButtonPanel = new Panel();
		ButtonPanel.setLayout(new GridLayout(1, 3,10,10));
		CurrentButton = new Button("Current");
		HistoryButton= new Button("Cumulated secs");
		History60Button= new Button("Cumulated mins");
		CohereButton = new Button("Coherence");
		CurrentButton.setForeground(Color.white);
		CurrentButton.setBackground(Color.darkGray);
		History60Button.setForeground(Color.white);
		History60Button.setBackground(Color.darkGray);

		HistoryButton.setForeground(Color.white);
		HistoryButton.setBackground(Color.darkGray);
		CohereButton.setForeground(Color.white);
		CohereButton.setBackground(Color.darkGray);
		ButtonPanel.add(CurrentButton);
		ButtonPanel.add(HistoryButton);
		ButtonPanel.add(History60Button);
		ButtonPanel.add(CohereButton);

		this.add("North", TimePanel);
		this.add("Center", MainPanel);
		this.add("South",  ButtonPanel);

        // dummy panel for marging left and right
        Panel d1 = new Panel();
        Panel d2 = new Panel();

        this.add("West", d1);
        this.add("East", d2);

		((CardLayout)MainPanel.getLayout()).show(MainPanel,"three");
}
	// start main thread
	public void start(){
		if(debug) new debug("start entered");

		thread = new Thread(this);
   		thread.start();
	}
	// stop all running threads
	public void stop(){

		if(debug) new debug("stop entered");
		if(thread != null ) thread.stop();
   		thread=null;
	}
	// main routine
	public void run(){

		int sleeptime = 950;

		// then enter the run

		URL sound_url=null;
		AudioClip soundtrack = null;
		try {
				sound_url = new URL( this.getDocumentBase(), "bgmusic.au");
		}
		catch (MalformedURLException e) {};
		soundtrack = getAudioClip(sound_url);
		soundtrack.loop();

		startDate = new Date();
		// always ask for data that starts 5 minutes before the current UTC
		startDate.setTime( startDate.getTime() + (startDate.getTimezoneOffset()*1000*60) - (1000*60*5));

		Date d = startDate;

		try {

			while(true){
					// always ask for data that ends 4 minutes before the current UTC
					endDate = new Date();
					endDate.setTime( endDate.getTime() + (endDate.getTimezoneOffset()*1000*60) - (1000*60*4));

					getEggData( d, endDate);

					// adjust animation speed to lag
					if( (endDate.getTime() - d.getTime()) > (60*1000) ) {
						if( sleeptime > 400) sleeptime -= 100;
					}

					if( (endDate.getTime() - d.getTime()) < (50*1000) ) {
						sleeptime += 100;
					}

					// last date is not used (bug in ICom class)
					endDate.setTime(endDate.getTime()-(1000));

					while(true) {

						DrawTimeString(d);
						if( curr_display !=  PAINT_COHERENCE){
							for( Enumeration e = eggs.keys() ; e.hasMoreElements() ; ){
								String lbl = (String)e.nextElement();
								EGG tmp = (EGG) eggs.get(lbl);
								tmp.animate(d);
							}
							for( Enumeration e = eggs.keys() ; e.hasMoreElements() ; ){
								String lbl = (String)e.nextElement();
								EGG tmp = (EGG) eggs.get(lbl);
								tmp.reveal();
							}
						}
						else {
							DrawCoherence(d);
						}
						d.setTime(d.getTime()+1000);

						//if( d.equals(endDate) ) break; // does not work anymore ???? enddate and startdate have hidden diffs
						if( d.getTime() - endDate.getTime() >= 0 ) break;

						try { Thread.sleep(sleeptime); } catch (InterruptedException e) {};
				}

				}
			}
		finally {
			soundtrack.stop();
		}
	}

	public void DrawCoherence(Date d){
		if(debug) new debug("BasketObserver: DrawCoherence()");


		//Graphics g = CoherePanel.getGraphics();
		Dimension size = CoherePanel.size();

		Image offscreen = CoherePanel.createImage(size.width,size.height);
		Graphics g = offscreen.getGraphics();

		// calc mean
		double sum=0;
		for( Enumeration e = eggs.keys() ; e.hasMoreElements() ; ){
			String lbl = (String)e.nextElement();
		 	EGG tmp = (EGG) eggs.get(lbl);
			EGGSample sample = (EGGSample) tmp.data.get(d.toString());
			if(  sample.rawData != missingVal ) sum +=  sample.rawData;
		}
		sum = sum/eggs.size();

		// calc sum of differences to mean
		double sumdiff=0;
		for( Enumeration e = eggs.keys() ; e.hasMoreElements() ; ){
			String lbl = (String)e.nextElement();
		 	EGG tmp = (EGG) eggs.get(lbl);
			EGGSample sample = (EGGSample) tmp.data.get(d.toString());
			if(  sample.rawData != missingVal ) sumdiff +=  Math.pow(  (double) (sum-((float) sample.rawData) ), 2 );
		}
		sumdiff = Math.sqrt(sumdiff);

		// coherence is inversely related to sum of differences
		sumdiff = (sumdiff == 0 ? size.width - 50: (1/sumdiff) * 100);
		if( sumdiff> (size.width-50) ) sumdiff = size.width-50;

		int value= (int) ((sumdiff));

		g.setColor(bgColor);
		g.fillRect(0,0,size.width,size.height);

		int x = Math.abs(value);
   		Color c = new Color ( 	(int) x < 9 ? 75 + (int) x*20 : 255,
   								(int) x < 16 ? 0 : ( (int) x < 25 ? 75 + (int) (x-16)*20 : 255 ),
   								0
   					  		);
		g.setColor(c);

		g.fillOval( (int) (size.width/2) - (int) (value/2), (int) (size.height/2)- (int) (value/2), value, value);

		g.setColor(Color.black);
	    float displayValue = Math.round(sumdiff * 1000);
        displayValue = displayValue / 1000;
  		g.drawString( "Coherence = " + Float.toString( (float) displayValue), 10, 10 );

		Graphics g2 = CoherePanel.getGraphics();
		g2.drawImage(offscreen, 0,0,this);
	}

	public void DrawTimeString(Date d){
		Graphics g = TimePanel.getGraphics();
		Dimension size = TimePanel.size();
		g.setColor(bgColor);
		g.fillRect(0,0,size.width,size.height);

		g.setColor(fgColor);
		g.drawString( d.toString() + " UTC" , 10, 10);
	}
	////////////////////////////////////////// Event methods
	public boolean action(Event event, Object arg) {

		if( event.target instanceof Button){

			if( event.target == CurrentButton ) {
				if( curr_display == PAINT_COHERENCE){
					((CardLayout)MainPanel.getLayout()).show(MainPanel,"one");
				}
				setDisplayType(PAINT_CURRENT);
				return true;
			}

			if( event.target == HistoryButton ) {
				if( curr_display == PAINT_COHERENCE){
					((CardLayout)MainPanel.getLayout()).show(MainPanel,"one");
				}
				setDisplayType(PAINT_HISTORY);
				return true;
			}

			if( event.target == History60Button ) {
				if( curr_display == PAINT_COHERENCE){
					((CardLayout)MainPanel.getLayout()).show(MainPanel,"one");
				}
				setDisplayType(PAINT_HISTORY60);
				return true;
			}

			if( event.target == CohereButton ) {
				((CardLayout)MainPanel.getLayout()).show(MainPanel,"two");
				curr_display = PAINT_COHERENCE;
				return true;
			}

			return true;
		}
		else return false;
	}

	public void setDisplayType(int type){
		for( Enumeration e = eggs.keys() ; e.hasMoreElements() ; ){
				String lbl = (String)e.nextElement();
				EGG tmp = (EGG) eggs.get(lbl);
				tmp.setDisplayType(type);
		}
		curr_display = type;
	}

	public void initColors () {
		if(debug) new debug("BasketObserver: initColors entered");
		bgColor = Color.white;
		this.setBackground(bgColor);
		fgColor = Color.black;
		this.setForeground(fgColor);
	}
	// dataretrieval
	public void getEggData(Date startDate, Date endDate){

		Date d = new Date(startDate.getTime());
		// construct searchargs
		int Day = d.getDate();
		int Month =  d.getMonth() + 1;
		int Year = 1900 +  d.getYear();
		int Hour = d.getHours();
		int Minute = d.getMinutes();
		int Second = d.getSeconds();

		// from time
		String stime = Hour<10 ? "0" + Integer.toString(Hour) : Integer.toString(Hour);
		stime = stime + "%3A" + ( Minute<10 ? "0" + Integer.toString(Minute) : Integer.toString(Minute) );
		stime = stime + "%3A" + ( Second<10 ? "0" + Integer.toString(Second) : Integer.toString(Second) );

		// to time
		d.setTime(endDate.getTime());
		Hour = d.getHours();
		Minute = d.getMinutes();
		Second = d.getSeconds();

		String etime = Hour<10 ? "0" + Integer.toString(Hour) : Integer.toString(Hour);
		etime = etime + "%3A" + ( Minute<10 ? "0" + Integer.toString(Minute) : Integer.toString(Minute) );
		etime = etime + "%3A" + ( Second<10 ? "0" + Integer.toString(Second) : Integer.toString(Second) );

		String SearchArgs = "";
		SearchArgs= SearchArgs.concat("z=1&year=" + Integer.toString(Year) + "&month=" + Integer.toString(Month) + "&day=" + Integer.toString(Day));
		SearchArgs= SearchArgs.concat("&stime=" + stime + "&etime=" +etime + "&idate=Yes");

		ICom netcon = new ICom(BasketAdress, SearchArgs, "");
		String fb = netcon.execute();


		parse(fb, startDate);
	}

	public void parse(String fb, Date startDate){


		// remove old data from eggs (to prevend memory overflow)
		for( Enumeration e = eggs.keys() ; e.hasMoreElements() ; ){

				String lbl = (String)e.nextElement();
				EGG tmpEgg3 = (EGG) eggs.get(lbl);

				Date tmpd = new Date( startDate.getTime()-1000 );
				EGGSample tmpsample = (EGGSample) tmpEgg3.data.get(tmpd.toString());

				tmpEgg3.data.clear();

	  			tmpEgg3.data.put(tmpd.toString(), tmpsample );
	}


		// set ,, in data to ,0, to assure data integrity
		String tmps = replace(fb, ",,", "," + missingVal + ",");
		fb = tmps;

		// make array of raw data
		StringTokenizer t = new StringTokenizer(fb, ",");
		int ndata = t.countTokens();
		String Data[] = new String[ndata];
		for(int i=0; i<ndata; i++)
			{ Data[i] = t.nextToken(); }
		t = null;

		// find start of Eggnumbers (data before Eggnumbers are dismissed for now)
		int i=0;
		String tmp = new String("\"Date/Time\"");
		while( ! (Data[i++].equals(tmp) ) ) {};
		// save Eggnumbers
		tmp = new String("eol");
		int nrOfEggs = 0;
		String eggnumbers[] = new String[100];
		while( ! (Data[i].equals(tmp) ) ) {
			eggnumbers[nrOfEggs++] = Data[i];
			i++;
		}

		// check if screen display needs update
		boolean updateScreen = false;
		for( int eggnr=0; eggnr<nrOfEggs; eggnr++){
			if( !eggs.containsKey( eggnumbers[eggnr] ) ){
				updateScreen=true;
				break;
			}
		}
		if( !updateScreen ){
			for( Enumeration e = eggs.keys() ; e.hasMoreElements() ; ){
				String lbl2 = (String)e.nextElement();
				boolean removeit = true;
				for( int eggnr=0; eggnr<nrOfEggs; eggnr++){
				if( lbl2.equals(eggnumbers[eggnr]) ) {
					removeit=false;
					break;
				}
			}
			if( removeit) {
				updateScreen= true;
				break;
			}
			}
		}

		if( updateScreen ){
			((CardLayout)MainPanel.getLayout()).show(MainPanel,"three");
		}


		// remove displays for eggs not in egglist
		for( Enumeration e = eggs.keys() ; e.hasMoreElements() ; ){

				String lbl = (String)e.nextElement();

			boolean removeit = true;
			for( int eggnr=0; eggnr<nrOfEggs; eggnr++){
				if( lbl.equals(eggnumbers[eggnr]) ) {
					removeit=false;
					break;
				}
			}
			if( removeit) {
				EGG tmpegg = (EGG) eggs.get(lbl);
				EggPanel.remove(tmpegg);
				eggs.remove(lbl);
			}
		}

		// make displays for eggs
		for( int eggnr=0; eggnr<nrOfEggs; eggnr++){

			if( !eggs.containsKey(eggnumbers[eggnr]) ){
				EGG  newEgg  =  new EGG(eggnumbers[eggnr], this);
				eggs.put(eggnumbers[eggnr], newEgg);
				EggPanel.add(newEgg);
			}
		}

		// fill the eggs
		i+=4;
		int j=0;
		int k=1;
		int eggnr = 0;
		Date d = new Date( startDate.getTime() );
		while( i< Data.length ) {
			while( ! (Data[i].equals(tmp) ) ){
				EGG tmpEgg2 = (EGG) eggs.get(eggnumbers[eggnr]);
				Date olddate = new Date(d.getTime()-1000);
				if( !tmpEgg2.data.containsKey( olddate.toString() ) )
					olddate = null;

				tmpEgg2.data.put(d.toString(), new EGGSample( Integer.parseInt(Data[i]) , 200, olddate == null ? null :(EGGSample) tmpEgg2.data.get(olddate.toString())) );

				eggnr++;
				if(eggnr==nrOfEggs) eggnr = 0;
				i++;
			}
			i+=4; j=0; k++; d.setTime(d.getTime()+1000);
		}



		if( updateScreen ){
			if( curr_display == PAINT_COHERENCE)
				((CardLayout)MainPanel.getLayout()).show(MainPanel,"two");
			else
				((CardLayout)MainPanel.getLayout()).show(MainPanel,"one");
		}
	}

public String replace(String in, String pat, String replacePat){
	String tmp = new String(in);
	int index= 0;
	index = tmp.indexOf(pat);

	while ( ( index = tmp.indexOf(pat)) != -1){
		tmp = tmp.substring(0,index) + replacePat + tmp.substring(index+2);
	}
	return tmp;
}

	//{{DECLARE_CONTROLS
	//}}
}



